#===============================================================================#
# Program to fit the accelerated failure time model with random effect (AFTMRE) #
# considering simulated data with censoring using R (version 3.2.1, 2015)       #
#===============================================================================#
rm(list=ls())

# Description of the data

# time: Vector of time
# delta: Indicator of failures (delta=1)
# x: Vector of covariates
# cod: Indicator of units

# Example: datasets with 30% and 50% of censoring
data <- read.delim("D:/dados/data30.txt", dec=",")
#data <- read.delim("E:/dados/data50.txt", dec=",")

attach(data)
View(data)
require(survival)
require(MASS)
require(Matrix)
require(car)
require(gamlss.dist)
require(lattice)

#===========================================#
# 1) Procedures for imputation - Method ITM #
#===========================================#


# Fit model considering the penalized log-likelihood to obtain preliminary estimates 
# of the parameters (EBLUE) and predictors of the random effects (EBLUP).

ajuste <- survreg(formula=Surv(time, delta) ~ x + frailty.gaussian(cod, sparse=TRUE), data)


X <- survreg(formula=Surv(time, delta) ~ x + frailty.gaussian(cod, sparse=TRUE), data,x = T)$x

X<-X[1:nrow(X),c(1:2)]

dim(X)

# Estimates of the fixed parameters
EBLUE<-matrix(ajuste$coef)
dim(EBLUE)

# Predictors of the random effects
EBLUP<-matrix(ajuste$frail)
dim(EBLUP)

# Parameter of scale
sigma=ajuste$scale


# Matrix Z
n=table(cod)
M=NULL
for (i in 1:(max(data$cod)-1)) 
M=c(M,rep(1,n[i]),rep(0,nrow(data)))
M=c(M,rep(1,n[max(data$cod)]))
Z=matrix(M,nrow(data),max(data$cod))

dim(Z)

# Diagonal matrix 
In=diag(rep(1,nrow(data)))

### Inverse of the distribution function of Tij given the vector of random effects bi, the vector of covariates xij and Tij > cij

N<-nrow(data) # Number of lines
gama<-1/sigma # Shape parameter
unif<-runif(N,0,1) # generates random number of standart uniform distribution

v_ger=function(x,time,cens,beta,b,gama,unif){
  xbeta<-x%*%beta   # beta: vector of fixed parameters 
  bi<-Z%*%EBLUP     # Vector of random effects
  tgenerated<-exp(bi+xbeta)*(-log(1-unif)+(time/(exp(bi+xbeta)))^(gama))^(1/gama)
  return(tgenerated)
}

### Generated values for all times

v_generated<-v_ger(x=X,time=data$time,cens=data$delta,beta=EBLUE,b=bi,gama=gama,unif=unif)

### Imputing censored times

t_imput<-function(v_ger2){
  for(i in 1:N){
    time_imput0<-ifelse(data$delta==0,v_generated,data$time)
  }
  return(time_imput0)
}

### Vector of imputed times 

time_imput<-t_imput(v_generated)


### Example of with create a new database imputed times

data_imput_ITM<-cbind(data,time_imput)
View(data_imput_ITM)

write.table(data_imput_ITM,file="data_imput_ITM.txt",sep="\t")

##########################################################################

#=================================================#
# 2) EBLUE and EBLUP considering the imputed data #
#=================================================#

# Data imputation with 30% of censoring
data_imput <- read.delim("~/data30_ITM.txt", dec=",")
attach(data_imput)
View(data_imput)

# Data imputation with 50% of censoring
data_imput <- read.delim("~/data50_ITM.txt", dec=",")
attach(data_imput)
View(data_imput)


y=log(time_imput)

# Matrix Z
n=table(cod)
M=NULL
for (i in 1:(max(data$cod)-1)) 
  M=c(M,rep(1,n[i]),rep(0,nrow(data)))
M=c(M,rep(1,n[max(data$cod)]))
Z=matrix(M,nrow(data),max(data$cod))

dim(Z)

# Diagonal matrix 
In=diag(rep(1,nrow(data)))

#===========================================================#
# Program to find the I-MINQUE of variance components       #
#===========================================================#

# Initial estimates (``seeds'')
theta0=0
sigma0=ajuste$scale 
sigma2_e=(pi^2/6)*sigma0^2

# V matrix estimated (Vo)
Vo=theta0*Z%*%t(Z)+(pi^2/6)*sigma2_e*In #(pi^2/6): vari^ancia da VEP

# Po matrix
Po=solve(Vo)-solve(Vo)%*%X%*%(solve(t(X)%*%solve(Vo)%*%X))%*%t(X)%*%solve(Vo)

# Sum of squares of all the elements of the matrices
s_oo=sum(Po^2)
s_1o=sum((Po%*%Z)^2)
s_11=sum((t(Z)%*%Po%*%Z)^2)
u_o=sum((Po%*%y)^2)
u_1=sum((t(Z)%*%Po%*%y)^2)

# Estimates of the variance components obtained by MINQUE method
theta=(s_oo*u_1-s_1o*u_o)/(s_oo*s_11-s_1o^2)
sigma2_e=(u_o-s_1o*theta)/s_oo

# Matrix of variance and covariance of Y (V)
V=theta*Z%*%t(Z)+sigma2_e*In #t(Z)  a transposta de Z
dim(V)

# EBLUE 
EBLUE=(solve(t(X)%*%solve(V)%*%X))%*%t(X)%*%solve(V)%*%y

#Matriz Of variance and covariance of EBLUE
var=solve(t(X)%*%solve(V)%*%X) 

# Standart error (SE) of EBLUE
EP=sqrt(diag(var)) 

# Covariance b and Y
C=theta*t(Z)

# EBLUP
EBLUP=C%*%solve(V)%*%(y-X%*%EBLUE)
dim(EBLUP)

##############################################################################################

#=============================================================#
# 3) Residual analysis for accelerated failure time models    # 
#                    with random effects                      #
#=============================================================#

N<-length(y)    # Number of observations
subject<-as.numeric(unique(cod))
k<-length(as.numeric(names(table(cod)))) ## Number of units
ni<-(table(cod)) # vector with number of observation by unit
p<-ncol(X)    # Number of location parameters

#obs=numeric()

#for(i in 1:n)
#{
#  obs=append(obs,1:vecni[i])                       # Observation labels for each unit
#}

# inverse of V
M<-solve(V) 

## Construction of the Q matrix 
VarBeta<-solve(t(X)%*%M%*%X)
Q<-M-M%*%X%*%(VarBeta)%*%t(X)%*%M
zq<- t(Z)%*%Q
norm.frob.ZtQ<-sum(diag(zq%*%t(zq)))

## Residual analysis

predm<-X%*%EBLUE                   # Predicted values for expected response
predi<-X%*%EBLUE+Z%*%EBLUP         # Predicted values for units
resm<-(y-predm) #+sigma*0.5772     # Marginal residuals
resc<-(y-predi)                    # Conditional residuals

#########################################################################

## This function obtains the square root of a matrix

sqrt.matrix <- function(mat) {              
  mat <- as.matrix(mat)  # new line of code
  singular_dec<-svd(mat,LINPACK=F)
  U<-singular_dec$u
  VI<-singular_dec$v
  D<-diag(singular_dec$d)
  sqrtmatrix<-U%*%sqrt(D)%*%t(VI)
  #  return(list(sqrt=sqrtmatrix))
}


## Variance of marginal residuals

var.resm<-V-X%*%solve(t(X)%*%M%*%X)%*%t(X)

## Standardized marginal residuals 

resmp<-resm/sqrt(diag(var.resm))

## Variance of conditional residuals

R<-sigma2_e^4*In # Varincia a partir do MINQUE
var.resc<- R%*%Q

## Standardized conditional residuals

rescp<- resc/sqrt(diag( var.resc))

# Total number of random effects
q<-length(EBLUP)/k 

## Mahalanobis distance 

aux=theta*t(Z)%*%Q%*%Z*theta

qm<-q-1
dm<-matrix(0,k,1)

for(j in 1:k) 
{
  if(q==1)
  {
    gbi<-aux[j,j]
    eblupi<-EBLUP[(q*j-qm):(q*j)]
    dmi<- t(eblupi)%*%solve(gbi)%*%eblupi
    dm[j]<-dmi
  }
  else
  {
    gbi<-aux[(q*j-qm):(q*j),(q*j-qm):(q*j)]
    eblupi<-EBLUP[(q*j-qm):(q*j)]
    dmi<- t(eblupi)%*%solve(gbi)%*%eblupi
    dm[j]<-dmi
  }
}

LSdm<- as.numeric(quantile(dm,prob=0.75) + 1.5*(quantile(dm,prob=0.75) - quantile(dm,prob=0.25)))
LS2dm<- 2*mean(dm)

## Modified Lesaffre-Verbeke index

lesverb<- rep(0,k) 
auxni=as.vector(ni)
for (t in 1:k){ 
  li<- sum(ni[1:t-1])+1
  ls<- sum(ni[1:t])
  if(ni[t]==1){
    
    auxr2 <- solve(sqrt(var.resm[li:ls,li:ls]))
    Ri<-(auxr2)%*%resm[li:ls]
    auxt<- diag(ni[t])-Ri%*%t(Ri)
    lesverb[t]<- sqrt(sum(diag(auxt%*%t(auxt))))
  }
  else
  {  
    
    auxr2 <- solve(sqrt.matrix(var.resm[li:ls,li:ls]))
    Ri<- auxr2%*%resm[li:ls]
    auxt<- diag(ni[t])-Ri%*%t(Ri)
    lesverb[t]<- sqrt(sum(diag(auxt%*%t(auxt))))
  } 
}

lesverb<-lesverb/((as.numeric((table(cod)))))
LSlesverb<- as.numeric(quantile(lesverb,prob=0.75) + 1.5*(quantile(lesverb,prob=0.75) - quantile(lesverb,prob=0.25)))
LS2lesverb<-2*mean(lesverb)

#################  FIM DAS CONTAS  #################

###########################################################
##########    Residual plots - English Legend   ###########
###########################################################

## QQplot2

qqPlot2<-function(x, distribution="norm", ..., ylab=deparse(substitute(x)),
                  xlab=paste(distribution, "quantiles"), main=NULL, las=par("las"),
                  envelope=.95,  
                  col=palette()[1], col.lines=palette()[2], lwd=2, pch=1, cex=par("cex"),
                  cex.lab=par("cex.lab"),cex.axis=par("cex.axis"), xlim=par("xlim"), ylim=par("ylim"), 
                  line=c("quartiles", "robust", "none"), 
                  labels = if(!is.null(names(x))) names(x) else seq(along=x),
                  id.method = "y", 
                  id.n = if(id.method[1]=="identify") Inf else 0,
                  id.cex=1, id.col=palette()[1], grid=TRUE)
{
  line <- match.arg(line)
  good <- !is.na(x)
  ord <- order(x[good])
  ord.x <- x[good][ord]
  ord.lab <- labels[good][ord]
  q.function <- eval(parse(text=paste("q", distribution, sep="")))
  d.function <- eval(parse(text=paste("d", distribution, sep="")))
  n <- length(ord.x)
  P <- ppoints(n)
  z <- q.function(P, ...)
  plot(z, ord.x, type="n", xlab=xlab, ylab=ylab, main=main, las=las,cex.lab=cex.lab, cex.axis=cex.axis, xlim=xlim, ylim=ylim)
  if(grid){
    grid(lty=1, equilogs=FALSE)
    box()}
  points(z, ord.x, col=col, pch=pch, cex=cex)
  if (line == "quartiles" || line == "none"){
    Q.x <- quantile(ord.x, c(.25,.75))
    Q.z <- q.function(c(.25,.75), ...)
    b <- (Q.x[2] - Q.x[1])/(Q.z[2] - Q.z[1])
    a <- Q.x[1] - b*Q.z[1]
    abline(a, b, col=col.lines, lwd=lwd)
  }
  if (line=="robust") {
    coef <- coef(rlm(ord.x ~ z))
    a <- coef[1]
    b <- coef[2]
    abline(a, b)
  }
  conf <- if (envelope == FALSE) .95 else envelope
  zz <- qnorm(1 - (1 - conf)/2)
  SE <- (b/d.function(z, ...))*sqrt(P*(1 - P)/n)
  fit.value <- a + b*z
  upper <- fit.value + zz*SE
  lower <- fit.value - zz*SE
  if (envelope != FALSE) {
    lines(z, upper, lty=2, lwd=lwd, col=col.lines)
    lines(z, lower, lty=2, lwd=lwd, col=col.lines)
  }
  showLabels(z, ord.x, labels=ord.lab,
             id.method = id.method, id.n = id.n, id.cex=id.cex, id.col=id.col)
}


# 1- Standardized marginal residuals versus fitted values and corresponding histogram

par(mfrow=c(1,1), mar=c(6, 7, 1, 2))
plot(predm, resmp, xlab=expression(paste("Marginal fitted values")),
     ylab=expression(paste("Standardized marginal residuals")), pch=20, cex=1.2, cex.lab=1.5, cex.axis=1.3,ylim=c(-6,6),xlim=c(4,11))
#ylim=c(-1.3*max(abs(range(resmp))),1.3*max(abs(range(resmp))))
LUresmp=quantile(resmp,prob=0.995)
LLresmp=quantile(resmp,prob=0.005)
abline(h=0, lty=2)
abline(h=LLresmp, lty=2)
abline(h=LUresmp, lty=2)
#index=which(abs(resmp)>limit)
#index=which(resmp > LUresmp | resmp < LLresmp)
#if(length(index)>0)
#{
 # text(predm[index], resmp[index], paste(cod[index], sep="."), adj=c(1,-.5), cex=1.0, font=2)
#}

hist(resmp, freq=F, main="", xlab=expression(paste("Standardized marginal residuals")), cex=0.9, cex.lab=1.5, cex.axis=1.3, xlim=c(-5,5))

# 2- Chi-squared QQ plot for Mahalanobis distance

par(mfrow=c(1,1),mar=c(6, 7, 1, 2))
quant.chisq<-qqPlot2(dm, distribution='chisq', df=q, pch=20, cex=1.2,cex.lab=1.5,cex.axis=1.3,xlim=c(0,8),ylim=c(0,10),
                     ylab=expression(paste("Mahalanobis distance")),xlab="Chi-squared quantiles")

# 3- Modified Lesaffre-Verbeke index versus unit indices

par(mfrow=c(1,1),mar=c(6, 7, 1, 2))
plot(lesverb,ylab=expression(paste("Modified Lesaffre-Verbeke index")),
     xlab="Unit index", cex=1.2, cex.lab=1.5, cex.axis=1.3, pch=20, ylim=c(0,12))
#ylim=c(0,2*max(abs(range(lesverb))))
abline(h=LSlesverb,lty=2) #2*mean(lesverbp)
#abline(h=LS2lesverb,lty=3) #2*mean(lesverbp)
index=which(lesverb>LSlesverb)
index1<-subject[index]
if(length(index)>0)
{
  text(index, lesverb[index], index1, adj=c(1,-.5), cex=1.0, font=2)
}

# 4- Standardized conditional residuals versus fitted values and corresponding histogram

par(mfrow=c(1,1),mar=c(6, 7, 1, 2))
plot(predi, rescp, xlab=expression(paste("Predicted values")),
     cex=1.2, cex.lab=1.5, cex.axis=1.3, ylab=expression(paste("Standardized conditional residuals")), pch=20,xlim=c(4,12),ylim=c(-2,2))
#ylim=c(-max(abs(range(rescp))),max(abs(range(rescp))))
limitupper=qGU(0.995,0.5772*sigma,sigma)
limitless=qGU(0.005,0.5772*sigma,sigma)
abline(h=c(limitupper,0,limitless),lty=2)
#abline(h=0,lty=2)
#abline(h=limitupper,lty=2)
#abline(h=limitless,lty=2)
#index=which(rescp > limitupper | rescp < limitless)
#index1<-subject[index]
#if(length(index)>0)
#{
 # text(predi[index], rescp[index], paste(cod[index], sep="."), adj=c(1,-.5), cex=1.0, font=2)
#}

hist(rescp, freq=F, main="", xlab=expression(paste("Standardized conditional residuals")), cex=1.0, cex.lab=1.5, cex.axis=1.3)

# 5- Standardized conditional residuals versus indices of the observations

par(mfrow=c(1,1),mar=c(6, 7, 1, 2))
plot(seq(1,N),rescp,xlab=expression(paste("Indices of observations ")),
     cex=1.0,cex.lab=1.5,cex.axis=1.3,ylab=expression(paste("Standardized conditional residuals ", widehat(e)[ij])),pch=20,ylim=c(-max(abs(range(rescp))),max(abs(range(rescp)))))
limitupper=qGU(0.995,0.5772*sigma,sigma)
limitless=qGU(0.005,0.5772*sigma,sigma)
abline(h=c(limitless,0,limitupper),lty=2)
#abline(h=0,lty=2)
#abline(h=limit,lty=2)
#abline(h=-limit,lty=2)
#index=which(rescp > limitupper | rescp < limitless)
#if(length(index)>0)
#{
 # text(seq(1,N)[index],rescp[index],paste(cod[index], sep="."),adj=c(1,-.5),cex=.75,font=2)
#}


# 6- QQ-Plot para Valor Extremo Padro

par(mfrow=c(1,1),mar=c(6, 7, 1, 2))
mu<-sigma*0.5772
sigmma<-sigma
quant.gumbel<-qqPlot2(resc,distribution='GU',mu,sigmma, pch=20, cex=1.2,cex.lab=1.5,cex.axis=1.3,xlim=c(-8,2),ylim=c(-6,4),
                      ylab=expression(paste("Conditional residuals " , (widehat(e)[ij]))),xlab="Extreme value quantiles")
####################################################################################################################################
